package cz.kebrt.unitcheck.jpf;

import java.io.PrintStream;

import cz.kebrt.unitcheck.api.data.InstructionHolder;
import cz.kebrt.unitcheck.api.data.PathHolder;
import cz.kebrt.unitcheck.api.data.StepHolder;
import cz.kebrt.unitcheck.api.data.TransitionHolder;
import cz.kebrt.unitcheck.config.PathPrinterConfig;
import cz.kebrt.unitcheck.util.StringUtil;

/**
 * Prints JPF structures representing error traces (Path, Transition) to streams.
 *
 * @author michal
 */
public class PathPrinter {

    private static final String LINE_SEP = StringUtil.newString('-', 80);

    private final PathPrinterConfig config;

    /**
     * Constructs a new path printer.
     *
     * @param config configuration used for printing
     */
    public PathPrinter(PathPrinterConfig config) {
        this.config = config;
    }

    /**
     * Prints a path to the stream.
     *
     * @param path path to be printed
     * @param out stream where the path will be printed
     */
    public void printPath(PathHolder path, PrintStream out) {
        int i = 0;
        for (TransitionHolder t : path.getTransitions()) {
            if (i != 0) {
                out.println(LINE_SEP);
            }
            out.println(String.format("transition: #%d; thread: %d",
                    i++, t.getThreadIndex()));
            out.flush();
            printTransition(t, out);
        }
    }

    /**
     * Prints a single transition to the stream.
     *
     * @param transition transition to be printed
     * @param out stream where the transition will be printed
     */
    public void printTransition(TransitionHolder transition, PrintStream out) {
        StepHolder lastStep = null;

        for (StepHolder s : transition.getSteps()) {
            if (!s.equals(lastStep)) {
                if (s.getLine() != null) {
                    printLine(s, out);
                    lastStep = s;
                }
            }

            if (config.isPrintBytecode()) {
                InstructionHolder ih = s.getInstruction();
                if (!isJRECode(ih) || config.isTraceJRE()) {
                    if (!s.instructionPositionEquals(lastStep)
                            && ih.getMethodName() != null) {
                        out.print("    ");
                        if (ih.getClassName() != null) {
                            out.print(ih.getClassName());
                            out.print('.');
                        }
                        out.println(ih.getMethodName());
                    }
                    out.print("      ");
                    out.println(ih.getInstructionName());
                }
                lastStep = s;
            }
        }
    }

    private void printLine(StepHolder s, PrintStream out) {
        String line = s.getLine();
        String loc = s.getLocation().replace('\\', '.').replace('/', '.');
        if (!isJRECode(loc) || config.isTraceJRE()) {
            loc = loc.replaceAll("(.*\\.)(.*)(\\.java)(.*)", //$NON-NLS-1$
                    "$1$2\\.\\?\\($2$3$4\\)"); //$NON-NLS-1$
            out.print(loc);
            out.print(" : ");
            out.println(line.trim());
        }
    }

    private boolean isJRECode(String className) {
        if (className == null) {
            return false;
        }
        if (className.startsWith("java") || className.startsWith("sun")) {
            return true;
        }
        return false;
    }

    private boolean isJRECode(InstructionHolder instruction) {
        String cls = instruction.getClassName();
        if (cls == null) {
            return false;
        }
        return isJRECode(cls);
    }

}
