package cz.kebrt.unitcheck.api;

import cz.kebrt.unitcheck.api.data.CheckResult;
import cz.kebrt.unitcheck.api.data.CheckRunResult;
import cz.kebrt.unitcheck.api.data.TestMethod;

/**
 * Informs about execution progress of each test checked by UnitCheck.
 * <p>
 * The listener provides a mechanism for reporting what is happening in the checking
 * process. Four callbacks must be implemented. Each of them identifies a test method and
 * provides the results collected while checking the test method. The callbacks whose name
 * start with <code>checkRun</code> are used when the checking of a test method is started
 * or completely finished. The <code>checkFinished</code> callback is invoked every time a
 * particular test method is finished (Each test method may finish a number of times
 * because UnitCheck explores all possible execution paths.).
 * <p>
 * Implementations of this listeners are added to {@link cz.kebrt.unitcheck.api.UnitChecker}.
 */
public interface UnitCheckListener {

    /**
     * Called before checking of a test method is started.
     *
     * @param tm test which is about to be started
     */
    void checkRunStarted(TestMethod tm);

    /**
     * Called after the end of a test method was reached. Each test method may finish a
     * number of times because UnitCheck explores all possible execution paths.
     *
     * @param tm test which was finished
     * @param result result of checking (the result covers only the latest execution path)
     */
    void checkFinished(TestMethod tm, CheckResult result);

    /**
     * Called after checking of a test method was completely finished.
     *
     * @param tm test which was finished
     * @param result result of checking
     */
    void checkRunFinished(TestMethod tm, CheckRunResult result);

    /**
     * Called after checking of a test method was interrupted by a user.
     *
     * @param tm test which was interrupted
     * @param result result of checking
     */
    void checkRunInterrupted(TestMethod tm, CheckRunResult result);

}
